package com.muyu.cargateway.instance;

import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.*;
import com.aliyun.tea.TeaException;
import com.aliyun.teaopenapi.models.Config;
import com.aliyun.teautil.Common;
import com.aliyun.teautil.models.RuntimeOptions;
import com.muyu.cargateway.domain.AliInstance;
import com.muyu.common.redis.service.RedisService;
import lombok.extern.log4j.Log4j2;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @ Tool：IntelliJ IDEA
 * @ Author：CHX
 * @ Date：2024-09-28-21:10
 * @ Version：1.0
 * @ Description：调用ali的类
 * @author Lenovo
 */
@Log4j2
@Component
public class Sample implements ApplicationRunner , DisposableBean {

    @Autowired
    private RedisService redisService;

    public static final String ACCESS_KEY_ID = "LTAI5tDH3FyRx4PRr6anx2TL";
    public static final String ACCESS_KEY_SECRET = "xdQnX2tDattY50raNkUWmHzE2tondP";

    public static Client createClient() throws Exception {
        // 工程代码泄露可能会导致 AccessKey 泄露，并威胁账号下所有资源的安全性。以下代码示例仅供参考。
        Config config = new Config()
                // 必填，请确保代码运行环境设置了环境变量 ALIBABA_CLOUD_ACCESS_KEY_ID。
                .setAccessKeyId(ACCESS_KEY_ID)
                // 必填，请确保代码运行环境设置了环境变量 ALIBABA_CLOUD_ACCESS_KEY_SECRET。
                .setAccessKeySecret(ACCESS_KEY_SECRET);
        // Endpoint 请参考 https://api.aliyun.com/product/Ecs
        config.endpoint = "ecs-cn-hangzhou.aliyuncs.com";
        return new Client(config);
    }

    /**
     * 创建实例
     * @throws Exception
     */
    public void generateInstance() throws Exception {
            Client client = Sample.createClient();
            RunInstancesRequest.RunInstancesRequestSystemDisk systemDisk = new RunInstancesRequest.RunInstancesRequestSystemDisk();
            //设置系统盘的大小为40GiB
            systemDisk.setSize("40");
            //设置系统盘类型为cloud_essd
            systemDisk.setCategory("cloud_essd");

            // 创建创建实例请求对象并设置参数

            RunInstancesRequest runInstancesRequest = new RunInstancesRequest()
                    // 设置地域ID
                    .setRegionId("cn-shanghai")
                    // 设置镜像ID
                    .setImageId("m-uf6ih0vnl5f51pquns11")
                    // 设置实例规格类型
                    .setInstanceType("ecs.t6-c1m1.large")
                    // 设置安全组ID
                    .setSecurityGroupId("sg-uf642d5u4ja5gsiitx8y")
                    // 设置虚拟交换机ID
                    .setVSwitchId("vsw-uf66lifrkhxqc94xi06v3")
                    // 设置实例名称
                    .setInstanceName("server-mqtt")
                    // 设置付费类型 按量付费
                    .setInstanceChargeType("PostPaid")
                    // 设置系统盘配置
                    .setSystemDisk(systemDisk)
                    // 设置用户名
                    .setHostName("root")
                    // 设置密码
                    .setPassword("10160810@a")
                    // 设置要创建的实例数量
                    .setAmount(2)
                    .setInternetChargeType("PayByTraffic")
                    .setInternetMaxBandwidthOut(1);

            RuntimeOptions runtimeOptions = new RuntimeOptions();
        // 尝试执行创建实例请求
        try {


            // 复制代码运行请自行打印 API 的返回值
            RunInstancesResponse runInstancesResponse = client.runInstancesWithOptions(runInstancesRequest, runtimeOptions);
            log.info("实例创建成功: {}", runInstancesResponse.getBody().getInstanceIdSets().instanceIdSet);
            RunInstancesResponseBody body = runInstancesResponse.getBody();
            RunInstancesResponseBody.RunInstancesResponseBodyInstanceIdSets instanceIdSets = body.getInstanceIdSets();

            List<AliInstance> instanceList = new ArrayList<>();
            List<String> instanceIds = new ArrayList<>();
            //遍历实例ID
            for (String string : instanceIdSets.instanceIdSet) {
                instanceIds.add(string);
            }
            // 在获取实例详细信息时，确保获取所有必要的信息
            DescribeInstancesResponse describeInstancesResponse = getInstances(client);
            List<DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance> instances = describeInstancesResponse.getBody().getInstances().getInstance();
            List<AliInstance> aliInstances = new ArrayList<>();
            instanceIds = new ArrayList<>();

            for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance instance : instances) {
                String instanceId = instance.getInstanceId();
                String ipAddress = instance.getPublicIpAddress().getIpAddress().get(0);
                String status = instance.getStatus();
                log.info("获取到的实例ID为{}  IP为{}  状态为{}", instanceId, ipAddress, status);
                aliInstances.add(new AliInstance(instanceId, ipAddress, status));
                instanceIds.add(instanceId);
            }
            //实例信息集合(实例id,ip 状态) 存储在redis中
            redisService.setCacheList("instanceList", aliInstances);
            // 实例ID集合 存储在redis中
            redisService.setCacheList("instanceIds", instanceIds);

        } catch (TeaException error) {
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            log.info(error.getMessage());
            // 诊断地址
            log.info(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            log.info(error.getMessage());
            // 诊断地址
            log.info(error.getData().get("Recommend"));
            Common.assertAsString(error.message);
        }
    }
    /**
     * 获取实例ID
     * @param client
     * @return string类型的实例ID集合
     * @throws Exception
     */
    @NotNull
    private DescribeInstancesResponse getInstances(Client client) throws Exception {
        DescribeInstancesRequest request = new DescribeInstancesRequest();
        request.setInstanceName("server-mqtt");
        request.setRegionId("cn-shanghai");

        RuntimeOptions runtimeOptions = new RuntimeOptions();
        return client.describeInstancesWithOptions(request, runtimeOptions);
    }
    /**
     * 删除创建的实例
     * @throws Exception
     */
    public void deleteSample() throws Exception {
        Client client = Sample.createClient();
        DescribeInstancesRequest attributeRequest = new DescribeInstancesRequest();
        attributeRequest.setRegionId("cn-shanghai");

        RuntimeOptions runtimeOptions = new RuntimeOptions();

        DescribeInstancesResponse instancesWithOptions = client.describeInstancesWithOptions(attributeRequest, runtimeOptions);
        ArrayList<String> list = new ArrayList<>();
        DescribeInstancesResponseBody body = instancesWithOptions.getBody();
        for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance instance : body.instances.instance) {
            list.add(instance.getInstanceId());
        }
        log.info("list:"+list);
        DeleteInstancesRequest deleteInstancesRequest = new DeleteInstancesRequest();
        deleteInstancesRequest.setRegionId("cn-shanghai")
                .setDryRun(false)
                .setForce(true)
                .setTerminateSubscription(true)
                .setInstanceId(list);
        RuntimeOptions runtime = new RuntimeOptions();
        try{
            client.deleteInstancesWithOptions(deleteInstancesRequest, runtime);
        }catch (TeaException error) {
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            System.out.println(error.getMessage());
            // 诊断地址
            System.out.println(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            System.out.println(error.getMessage());
            // 诊断地址
            System.out.println(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        }
    }

    @Override
    public void run(ApplicationArguments args){
        try {
            log.info("开始创建实例");
            generateInstance();
        } catch (Exception e) {
            log.info("创建实例失败");
            throw new RuntimeException(e);
        }
        log.info("创建实例成功");
    }

    @Override
    public void destroy(){
        try {
            log.info("删除实例方法");
            Thread.sleep(10000);
            deleteSample();
        } catch (Exception e) {
            log.info("删除实例失败");
            throw new RuntimeException(e);
        }
        log.info("删除实例成功");
    }
}
